<?php
/**
 * @file
 *    Example server-side implementation for the jQuery.dashboard() plugin.
 *
 * Released under the GNU General Public License.  See LICENSE.txt.
 */

// Handles all requests.
function execute() {
  // Simulate slow network connections.
  sleep(rand(10, 50) * 0.05);

  // Find the operation of this request.
  $op = $GLOBALS['_GET']['op'] ? $GLOBALS['_GET']['op'] : ($GLOBALS['_POST']['op'] ? $GLOBALS['_POST']['op'] : false);

  // Call this operation's handler.
  if ($op and $ret = call_function_if_exists("execute_op_$op")) {
    print to_js($ret);
  }
}

// Calls a function if it exists and returns the function's return value.
function call_function_if_exists($func) {
  if (function_exists($func)) {
    return $func();
  }
  else {
    return "Function '$func' is not defined.";
  }
}

// Taken from http://api.drupal.org/api/function/drupal_to_js/7 (GPL 2)
function to_js($var) {
  // json_encode() does not escape <, > and &, so we do it with str_replace()
  return str_replace(array("<", ">", "&"), array('\x3c', '\x3e', '\x26'), json_encode($var));
}

// Finds the id parameter and includes the respective widget file.
function include_widget_file() {
  $id = $GLOBALS['_GET']['id'] ? $GLOBALS['_GET']['id'] : ($GLOBALS['_POST']['id'] ? $GLOBALS['_POST']['id'] : '');

  // IDs client-side use hyphen separators.  Server-side they use underscores.
  $id = str_replace('-', '_', $id);

  $filename = "widgets/$id.inc";
  if (file_exists($filename)) {
    include $filename;
    return $id;
  }
  else {
    return "You need to create a widget file called '$filename'.";
  }
}

// Operation handler for get_widgets_by_column operation.
function execute_op_get_widgets_by_column() {
  // This would normally be coming from either the database (this user's settings) or a default/initial dashboard configuration.
  $widgets = array(
    array('hello-world' => 0, 'hello-rebekah' => 1),
    array('hello-bevan' => 0),
    array('news' => 0),
  );

  // The serialized version as sent by client-side script.
  $widgets = unserialize('a:3:{i:0;a:1:{s:11:"hello-world";i:0;}i:1;a:1:{s:11:"hello-bevan";i:0;}i:2;a:2:{s:13:"hello-rebekah";i:1;s:4:"news";i:0;}}');

  return $widgets;
}

// Operation handler for get_widget operation.
function execute_op_get_widget() {
  $id = include_widget_file();
  return call_function_if_exists("widget_$id");
}

// Operation handler for save_columns operation.
function execute_op_save_columns() {
  $cols = $GLOBALS['_POST']['columns'];

  // Parse out strings "1" and "0" as ints/booleans.
  foreach ($cols as $c => $widgets) {
    foreach ($widgets as $wid => $is_minimized) {
      $cols[$c][$wid] = (int) $is_minimized;
    }
  }

  // A couple of different debug options.
  // return serialize($cols);
  print_r($cols);
}

// Operation handler for widget_settings operation.
function execute_op_widget_settings() {
  $id = include_widget_file();
  return call_function_if_exists("widget_{$id}_settings");
}

execute();
